@@ -8,6 +8,8 @@ from abc import ABC, abstractmethod
 import json
 import pickle
 import random
+import sys
+import logging
 import numpy as np
 import pandas as pd
 import matplotlib.pyplot as plt
@@ -21,6 +23,9 @@ import cma
 
 from cubnm import sim, utils
 
+logging.basicConfig(stream=sys.stdout, level=logging.WARNING)
+logger = logging.getLogger(__name__)
+
 # define evaluation metric labels used in plots
 METRIC_LABELS = {
     'cost': 'Cost',
@@ -52,7 +57,7 @@ class BNMProblem(Problem):
         Brain network model problem. A :class:`pymoo.core.problem.Problem` 
         that defines the model, free parameters and their ranges, and target empirical
         data (FC and FCD), and the simulation configurations (through 
-        :class:`cubnm.sim.SimGroup`). 
+        :class:`cubnm.sim.base.SimGroup`). 
         :class:`cubnm.optimize.Optimizer` classes can be
         used to optimize the free parameters of this problem.
 
@@ -121,7 +126,7 @@ class BNMProblem(Problem):
             (via summation) defines each objective separately. This must not be used
             with single-objective optimizers
         **kwargs
-            Keyword arguments passed to :class:`cubnm.sim.SimGroup`
+            Keyword arguments passed to :class:`cubnm.sim.base.SimGroup`
         """
         # TODO: break down __init__ into smaller methods
         # set opts
@@ -143,10 +148,10 @@ class BNMProblem(Problem):
         # and do_fc/do_fcd are set to True
         if emp_bold is not None:
             if (emp_fc_tril is not None) or (emp_fcd_tril is not None):
-                print(
-                    "Warning: Both empirical BOLD and empirical FC/FCD are"
-                    " provided. Empirical FC/FCD will be calculated based on"
-                    " BOLD and will be overwritten."
+                logger.warning(
+                    "Both empirical BOLD and empirical FC/FCD are "
+                    "provided. Empirical FC/FCD will be calculated based on "
+                    "BOLD and will be overwritten."
                 )
             if self.sim_group.do_fc:
                 self.emp_fc_tril = utils.calculate_fc(
@@ -195,12 +200,6 @@ class BNMProblem(Problem):
         # decide if parameters can be variable across nodes/groups
         # note that this is different from map-based heterogeneity
         self.is_node_based = self.node_grouping is not None
-        if self.is_node_based:
-            for param in self.het_params:
-                if not isinstance(params[param], tuple):
-                    raise ValueError(
-                        f"{param} is set to be heterogeneous based on groups but no range is provided"
-                    )
         # set up node_groups (ordered index of all unique groups)
         # and memberships (from node i to N, which group they belong to)
         if self.is_node_based:
@@ -211,8 +210,9 @@ class BNMProblem(Problem):
                     # is the only member of it
                     self.memberships = np.arange(self.sim_group.nodes)
                 elif self.node_grouping == "sym":
-                    print(
-                        "Warning: `sym` node grouping assumes symmetry of parcels between L and R hemispheres"
+                    logger.warning(
+                        "`sym` node grouping assumes symmetry "
+                        "of parcels between L and R hemispheres"
                     )
                     # nodes i & rh_idx+i belong to the same group
                     # and will have similar parameters
@@ -227,27 +227,36 @@ class BNMProblem(Problem):
                 raise ValueError(
                     "Invalid node_grouping provided."
                 )
+            assert self.memberships.shape == (self.sim_group.nodes,), (
+                    f"Invalid node_grouping shape: {self.memberships.shape}."
+                )
             self.node_groups = np.unique(self.memberships)
 
         # set up global and regional (incl. bias) free parameters
+        # note that the `params` dictionary may include parameters
+        # that indicate fixed values of how heterogeneity should
+        # be applied to regional parameters (e.g. 'w_pscale0' or 'w_p0')
         for param, v in params.items():
-            if isinstance(v, tuple):
-                if (
-                    self.is_node_based
-                    & (param in self.regional_params)
-                    & (param in self.het_params)
-                ):
-                    # set up regional parameters which are regionally variable based on groups
-                    for group in self.node_groups:
-                        param_name = f"{param}{group}"
-                        self.free_params.append(param_name)
+            if param in self.global_params + self.regional_params:
+                if isinstance(v, tuple):
+                    if (
+                        self.is_node_based
+                        & (param in self.regional_params)
+                        & (param in self.het_params)
+                    ):
+                        # set up regional parameters which are regionally variable based on groups
+                        for group in self.node_groups:
+                            param_name = f"{param}{group}"
+                            if param_name not in params.keys():
+                                self.free_params.append(param_name)
+                                self.lb.append(v[0])
+                                self.ub.append(v[1])
+                            
+                    else:
+                        # is a global or bias parameter
+                        self.free_params.append(param)
                         self.lb.append(v[0])
                         self.ub.append(v[1])
-                else:
-                    # is a global or bias parameter
-                    self.free_params.append(param)
-                    self.lb.append(v[0])
-                    self.ub.append(v[1])
         self.fixed_params = list(set(self.params.keys()) - set(self.free_params))
 
         # set up map scaler parameters
@@ -264,33 +273,36 @@ class BNMProblem(Problem):
             ), f"Maps second dimension {self.maps.shape[1]} != nodes {self.sim_group.nodes}"
 
             # define map coefficients as free parameter
+            # unless they are specified in the input `params`
+            # dictionary
             for param in self.het_params:
                 for map_idx in range(self.maps.shape[0]):
-                    if self.maps_coef_range == 'auto':
-                        # identify the scaler range
-                        map_max = self.maps[map_idx, :].max()
-                        map_min = self.maps[map_idx, :].min()
-                        if (map_min < 0) & (map_max > 0):
-                            # e.g. z-scored
-                            scale_min = -1 / map_max
-                            scale_min = np.ceil(scale_min / 0.1) * 0.1  # round up
-                            scale_max = -1 / map_min
-                            scale_max = np.floor(scale_max / 0.1) * 0.1  # round down
-                        else:
-                            scale_min = 0.0
-                            scale_max = 1.0
-                    elif isinstance(self.maps_coef_range, tuple):
-                        scale_min, scale_max = self.maps_coef_range
-                    elif isinstance(self.maps_coef_range, list):
-                        scale_min, scale_max = self.maps_coef_range[map_idx]
-                    else:
-                        raise ValueError("Invalid maps_coef_range provided")
-                    # add the scaler as a free parameter
                     scaler_name = f"{param}scale{map_idx}"
-                    self.free_params.append(scaler_name)
-                    self.lb.append(scale_min)
-                    self.ub.append(scale_max)
-
+                    if scaler_name not in params.keys():
+                        # add the scaler as a free parameter
+                        # determine the scaler range
+                        if self.maps_coef_range == 'auto':
+                            # identify the scaler range
+                            map_max = self.maps[map_idx, :].max()
+                            map_min = self.maps[map_idx, :].min()
+                            if (map_min < 0) & (map_max > 0):
+                                # e.g. z-scored
+                                scale_min = -1 / map_max
+                                scale_min = np.ceil(scale_min / 0.1) * 0.1  # round up
+                                scale_max = -1 / map_min
+                                scale_max = np.floor(scale_max / 0.1) * 0.1  # round down
+                            else:
+                                scale_min = 0.0
+                                scale_max = 1.0
+                        elif isinstance(self.maps_coef_range, tuple):
+                            scale_min, scale_max = self.maps_coef_range
+                        elif isinstance(self.maps_coef_range, list):
+                            scale_min, scale_max = self.maps_coef_range[map_idx]
+                        else:
+                            raise ValueError("Invalid maps_coef_range provided")
+                        self.free_params.append(scaler_name)
+                        self.lb.append(scale_min)
+                        self.ub.append(scale_max)
             # define regional ranges
             if self.het_params_range == "same":
                 self._het_params_range = {}
@@ -309,14 +321,8 @@ class BNMProblem(Problem):
                         raise ValueError(
                             f"Parameter {param} is not defined in params"
                         )
-            elif isinstance(self.het_params_range, dict):
-                assert all(
-                    p in self.het_params_range for p in self.het_params
-                ), "All het_params must be defined in het_params_range"
+            elif isinstance(self.het_params_range, dict) or (self.het_params_range is None):
                 self._het_params_range = self.het_params_range
-            elif self.het_params_range is None:
-                # do not normalize the regional parameters
-                self._het_params_range = None
 
         # convert bounds to arrays
         self.lb = np.array(self.lb)
@@ -358,7 +364,7 @@ class BNMProblem(Problem):
         ----------
         include_sim_group: :obj:`bool`
             whether to include the configuration of the
-            associated :class:`cubnm.sim.SimGroup`
+            associated :class:`cubnm.sim.base.SimGroup`
         include_N: :obj:`bool`
             whether to include the current population size
             in the configuration
@@ -426,7 +432,7 @@ class BNMProblem(Problem):
     def _get_sim_params(self, X):
         """
         Gets the global and regional parameters of the problem's 
-        :class:`cubnm.sim.SimGroup` based on the
+        :class:`cubnm.sim.base.SimGroup` based on the
         problem free and fixed parameters and type of regional parameter
         heterogeneity (map-based, group-based or none).
 
@@ -453,7 +459,7 @@ class BNMProblem(Problem):
                 param_lists[param] = np.repeat(
                     self.params[param], N
                 )
-            else:
+            elif param in self.regional_params:
                 param_lists[param] = np.tile(
                     self.params[param], (N, self.sim_group.nodes)
                 )
@@ -474,9 +480,15 @@ class BNMProblem(Problem):
                     param_scalers = np.ones(self.sim_group.nodes)
                     for map_idx in range(self.maps.shape[0]):
                         scaler_name = f"{param}scale{map_idx}"
-                        param_scalers += (
-                            self.maps[map_idx, :] * Xt.iloc[sim_idx].loc[scaler_name]
-                        )
+                        if scaler_name in self.fixed_params:
+                            param_scalers += (
+                                self.maps[map_idx, :] * self.params[scaler_name]
+                            )
+                        else:
+                            # free
+                            param_scalers += (
+                                self.maps[map_idx, :] * Xt.iloc[sim_idx].loc[scaler_name]
+                            )
                     # multiply it by the bias which is already put in param_lists
                     param_lists[param][sim_idx, :] *= param_scalers
                     # fix out-of-range parameter values if indicated
@@ -515,16 +527,19 @@ class BNMProblem(Problem):
                 curr_param_maps = np.zeros((Xt.shape[0], self.sim_group.nodes))
                 for group in self.node_groups:
                     param_name = f"{param}{group}"
-                    curr_param_maps[:, self.memberships == group] = Xt.loc[
-                        :, param_name
-                    ].values[:, np.newaxis]
+                    if param_name in self.fixed_params:
+                        curr_param_maps[:, self.memberships == group] = self.params[param_name]
+                    else:
+                        curr_param_maps[:, self.memberships == group] = Xt.loc[
+                            :, param_name
+                        ].values[:, np.newaxis]
                 param_lists[param] = curr_param_maps
         return param_lists
 
     def _set_sim_params(self, X):
         """
         Sets the global and regional parameters of the problem's 
-        :class:`cubnm.sim.SimGroup` based on the
+        :class:`cubnm.sim.base.SimGroup` based on the
         problem free and fixed parameters and type of regional parameter
         heterogeneity (map-based, group-based or none).
 
@@ -535,6 +550,8 @@ class BNMProblem(Problem):
             Shape: (N, ndim)
         """
         self.sim_group.param_lists.update(self._get_sim_params(X))
+        # set the other missing parameters to defaults
+        self.sim_group._set_default_params(missing=True)
 
     def _evaluate(self, X, out, *args, **kwargs):
         """
@@ -621,7 +638,7 @@ class Optimizer(ABC):
         Saves the output of the optimizer, including history
         of particles, history of optima, the optimal point,
         and its simulation data. The output will be saved
-        to `out_dir` of the problem's :class:`cubnm.sim.SimGroup`.
+        to `out_dir` of the problem's :class:`cubnm.sim.base.SimGroup`.
         If a directory with the same type of optimizer already
         exists, a new directory with a new index will be created.
 
@@ -1155,17 +1172,17 @@ class GridOptimizer(Optimizer):
 
         Example
         -------
-        Run a grid search of rWW model with 10 G and 10 wEE values
-        with fixed wEI: ::
+        Run a grid search of rWW model with 100 simulations running
+        a 4 by 5 by 5 grid search of parameters G, w_p and J_N: ::
 
             from cubnm import datasets, optimize
 
             problem = optimize.BNMProblem(
                 model = 'rWW',
                 params = {
-                    'G': (0.5, 2.5),
-                    'wEE': (0.05, 0.75),
-                    'wEI': 0.21,
+                    'G': (0.001, 10.0),
+                    'w_p': (0, 2.0),
+                    'J_N': (0.001, 0.5),
                 },
                 duration = 60,
                 TR = 1,
@@ -1175,7 +1192,7 @@ class GridOptimizer(Optimizer):
                 emp_bold = datasets.load_bold('schaefer-100'),
             )
             go = optimize.GridOptimizer()
-            go.optimize(problem, grid_shape=10)
+            go.optimize(problem, grid_shape={'G': 4, 'w_p': 5, 'J_N': 5})
             go.save()
         """
         pass
@@ -1220,7 +1237,7 @@ class GridOptimizer(Optimizer):
         Saves the output of the optimizer, including history
         of particles, history of optima, the optimal point,
         and its simulation data. The output will be saved
-        to `out_dir` of the problem's :class:`cubnm.sim.SimGroup`.
+        to `out_dir` of the problem's :class:`cubnm.sim.base.SimGroup`.
 
         Parameters
         ---------
@@ -1400,16 +1417,16 @@ class CMAESOptimizer(PymooOptimizer):
         Example
         -------
         Run a CMAES optimization for 10 iterations with 
-        a population size of 20: ::
+        a population size of 20 on a 3D model (rWW, homogeneous): ::
 
             from cubnm import datasets, optimize
 
             problem = optimize.BNMProblem(
                 model = 'rWW',
                 params = {
-                    'G': (0.5, 2.5),
-                    'wEE': (0.05, 0.75),
-                    'wEI': 0.15,
+                    'G': (0.001, 10.0),
+                    'w_p': (0, 2.0),
+                    'J_N': (0.001, 0.5),
                 },
                 emp_bold = datasets.load_bold('schaefer-100'),
                 duration = 60,
@@ -1424,10 +1441,10 @@ class CMAESOptimizer(PymooOptimizer):
         """
         super().__init__(**kwargs)
         if self.seed == 0:
-            print(
-                "Warning: Seed value for CMAES is set to 0."
-                " This will result in different random initializations for each run"
-                " due to how `cma` works. Setting seed to 100 instead."
+            logger.warning(
+                "Seed value for CMAES is set to 0. "
+                "This will result in different random initializations for each run "
+                "due to how `cma` works. Setting seed to 100 instead."
             )
             self.seed = 100
         self.use_bound_penalty = (
@@ -1529,9 +1546,9 @@ def batch_optimize(optimizers, problems, save=True, setup_kwargs={}):
         problem_kwargs = dict(
             model = 'rWW',
             params = {
-                'G': (1.0, 3.0),
-                'wEE': (0.05, 0.5),
-                'wEI': 0.15,
+                'G': (0.001, 10.0),
+                'w_p': (0, 2.0),
+                'J_N': (0.001, 0.5),
             },
             duration = 60,
             TR = 1,
