@@ -1,11 +1,14 @@
 """
 Utility functions
 """
-import numpy as np
-import scipy
 import subprocess
 import json
 import gc
+import sys
+import logging
+import numpy as np
+import scipy
+
 from cubnm._setup_opts import gpu_model_flag
 try:
     import cupy as cp
@@ -16,6 +19,9 @@ except ImportError:
     cuda = None
     has_cupy = False
 
+logging.basicConfig(stream=sys.stdout, level=logging.WARNING)
+logger = logging.getLogger(__name__)
+
 if has_cupy:
     @cuda.jit
     def cdf_2d_2d_right(A_sorted, x_values, out):
@@ -95,10 +101,10 @@ def avail_gpus():
         ((gpu_model_flag == 'nvidia') and (gpu_counts['rocm'] > 0)) |
         ((gpu_model_flag == 'rocm') and (gpu_counts['nvidia'] > 0))
     ):
-        print(
-            f"Warning: Toolbox compiled for {gpu_model_flag}"
-            f" but {(set(gpu_counts.keys()) - set([gpu_model_flag]))}"
-            " GPUs are available. Reinstall for the correct GPU model."
+        logger.warning(
+            f"Toolbox compiled for {gpu_model_flag} "
+            f"but {(set(gpu_counts.keys()) - set([gpu_model_flag]))} "
+            "GPUs are available. Reinstall for the correct GPU model."
         )
     # but only return the number of gpus for which
     # the code was compiled
@@ -106,7 +112,7 @@ def avail_gpus():
 
 def is_jupyter():
     """
-    This function checks if the current environment is a Jupyter notebook.
+    Check if the current environment is a Jupyter notebook
 
     Returns
     -------
