@@ -591,6 +591,12 @@ void _init_cpu(BaseModel *m, bool force_reinit) {
     // set up constants (based on dt)
     Model::init_constants(m->dt);
 
+    // keep track of number of simulations for which
+    // memory will be allocated to later free them accordingly
+    // (otherwise if changed in the next call, freeing memory
+    // will be done incorrectly and may access invalid memory)
+    m->alloc_N_SIMS = m->N_SIMS;
+
     // set up global int and bool outputs
     if (Model::n_global_out_int > 0) {
         m->global_out_int = (int**)malloc(Model::n_global_out_int * sizeof(int*));
@@ -665,7 +671,7 @@ void _init_cpu(BaseModel *m, bool force_reinit) {
         #ifndef NOISE_SEGMENT
         // precalculate the entire noise needed; can use up a lot of memory
         // with high N of nodes and longer durations leads maxes out the memory
-        m->noise_size = m->nodes * m->bw_it * m->inner_it * Model::n_noise; // *10 for 0.1msec
+        m->noise_size = m->nodes * m->bw_it * m->inner_it * Model::n_noise;
         #else
         // otherwise precalculate a noise segment and arrays of shuffled
         // nodes and time points and reuse-shuffle the noise segment
@@ -695,7 +701,7 @@ void _init_cpu(BaseModel *m, bool force_reinit) {
                 "timepoints [columns] will be shuffled in each repeat)" << std::endl;
         }
         m->shuffled_nodes = (int*)malloc(m->noise_repeats * m->nodes * sizeof(int));
-        m->shuffled_ts = (int*)malloc(m->noise_repeats * m->base_conf.noise_time_steps * sizeof(int));
+        m->shuffled_ts = (int*)malloc(m->noise_repeats * m->noise_bw_it * sizeof(int));
         get_shuffled_nodes_ts(&(m->shuffled_nodes), &(m->shuffled_ts),
             m->nodes, m->noise_bw_it, m->noise_repeats, &rand_gen);
         #endif
@@ -729,7 +735,7 @@ void BaseModel::free_cpu() {
     }
     if (this->alloc_states_out) {
         for (int var_idx=0; var_idx<this->get_n_state_vars(); var_idx++) {
-            for (int sim_idx=0; sim_idx<this->N_SIMS; sim_idx++) {
+            for (int sim_idx=0; sim_idx<this->alloc_N_SIMS; sim_idx++) {
                 free(this->states_out[var_idx][sim_idx]);
             }
             free(this->states_out[var_idx]);
