@@ -8,6 +8,7 @@
 #include <numpy/arrayobject.h>
 #include <vector>
 #include <map>
+#include <functional>
 #include <gsl/gsl_vector.h>
 #include <gsl/gsl_statistics.h>
 #include <gsl/gsl_matrix_double.h>
@@ -34,7 +35,6 @@
 #include <complex>
 #include <algorithm>
 #include <memory>
-#include <iomanip>
 #include <stdexcept>
 #include "cubnm/defines.h"
 #include "utils.cpp"
@@ -42,10 +42,11 @@
 #include "./models/base.cpp"
 #include "fc.cpp"
 #include "bnm.cpp"
-#include "./models/rww.cpp"
-#include "./models/rww_fic.cpp"
-#include "./models/rwwex.cpp"
-#include "./models/kuramoto.cpp"
+
+using std::max; // TODO: find a better fix; This is to be able to use "max" in model equations (similar to cuda)
+using std::min;
+
+#include "models.cpp"
 
 // create a pointer to the model object in current session
 // so that it can be reused in subsequent calls to run_simulations
@@ -100,7 +101,7 @@ std::map<std::string, std::string> dict_to_map(PyObject *config_dict) {
 }
 
 template<typename T>
-void array_to_np_3d(T *** arr, PyObject * np_arr) {
+void array_3d_to_np(T *** arr, PyObject * np_arr) {
     // converts a 3d array to a 3d numpy array
     int dim1 = PyArray_DIM(np_arr, 0);
     int dim2 = PyArray_DIM(np_arr, 1);
@@ -117,7 +118,7 @@ void array_to_np_3d(T *** arr, PyObject * np_arr) {
 }
 
 template<typename T>
-void array_to_np_2d(T ** arr, PyObject * np_arr) {
+void array_2d_to_np(T ** arr, PyObject * np_arr) {
     // converts a 2d array to a 2d numpy array
     int dim1 = PyArray_DIM(np_arr, 0);
     int dim2 = PyArray_DIM(np_arr, 1);
@@ -131,7 +132,7 @@ void array_to_np_2d(T ** arr, PyObject * np_arr) {
 }
 
 template<typename T>
-void array_to_np_1d(T * arr, PyObject * np_arr) {
+void array_1d_to_np(T * arr, PyObject * np_arr) {
     // converts a 1d array to a nd numpy array
     // Note that unlike the other 2d and 3d functions
     // here np_arr can be n-dimensional
@@ -263,25 +264,11 @@ static PyObject* _run_simulations(PyObject* self, PyObject* args) {
         if (model != nullptr) {
             delete model; // delete the old model to ensure only one model object exists
         }
-        if (strcmp(model_name, "rWW")==0) {
-            model = new rWWModel(
-                nodes, N_SIMS, N_SCs, BOLD_TR, states_sampling, 
-                time_steps, do_delay, sim_seed, dt, bw_dt
-            );
-        } 
-        else if (strcmp(model_name, "rWWEx")==0) {
-            model = new rWWExModel(
-                nodes, N_SIMS, N_SCs, BOLD_TR, states_sampling, 
-                time_steps, do_delay, sim_seed, dt, bw_dt
-            );
-        } 
-        else if (strcmp(model_name, "Kuramoto")==0) {
-            model = new KuramotoModel(
-                nodes, N_SIMS, N_SCs, BOLD_TR, states_sampling, 
-                time_steps, do_delay, sim_seed, dt, bw_dt
-            );
-        }
-        else {
+        auto it = model_factory.find(model_name);
+        if (it != model_factory.end()) {
+            model = it->second(nodes, N_SIMS, N_SCs, BOLD_TR, states_sampling, 
+                            time_steps, do_delay, sim_seed, dt, bw_dt);
+        } else {
             std::cerr << "Model " << model_name << " not found" << std::endl;
             Py_RETURN_NONE;
         }
@@ -358,7 +345,7 @@ static PyObject* _run_simulations(PyObject* self, PyObject* args) {
     npy_intp noise_dims[1] = {model->noise_size};
     #ifdef NOISE_SEGMENT
     npy_intp shuffled_nodes_dims[2] = {model->noise_repeats, model->nodes};
-    npy_intp shuffled_ts_dims[2] = {model->noise_repeats, model->base_conf.noise_time_steps};
+    npy_intp shuffled_ts_dims[2] = {model->noise_repeats, model->noise_bw_it};
     #endif
 
     PyObject *py_BOLD_ex_out, *py_fc_trils_out, *py_fcd_trils_out,
@@ -435,15 +422,15 @@ static PyObject* _run_simulations(PyObject* self, PyObject* args) {
 
     // Copy the output C arrays to NumPy arrays
     if (ext_out) {
-        array_to_np_3d<double>(model->states_out, py_states_out);
-        array_to_np_2d<bool>(model->global_out_bool, py_global_bools_out);
-        array_to_np_2d<int>(model->global_out_int, py_global_ints_out);
+        array_3d_to_np<double>(model->states_out, py_states_out);
+        array_2d_to_np<bool>(model->global_out_bool, py_global_bools_out);
+        array_2d_to_np<int>(model->global_out_int, py_global_ints_out);
     }
     if (noise_out) {
-        array_to_np_1d<double>(model->noise, py_noise_out);
+        array_1d_to_np<double>(model->noise, py_noise_out);
         #ifdef NOISE_SEGMENT
-        array_to_np_1d<int>(model->shuffled_nodes, py_shuffled_nodes_out);
-        array_to_np_1d<int>(model->shuffled_ts, py_shuffled_ts_out);
+        array_1d_to_np<int>(model->shuffled_nodes, py_shuffled_nodes_out);
+        array_1d_to_np<int>(model->shuffled_ts, py_shuffled_ts_out);
         #endif
     }
 
@@ -606,7 +593,7 @@ static PyMethodDef methods[] = {
         #ifdef NOISE_SEGMENT
         "shuffled_nodes (np.ndarray) (noise_repeats, nodes)\n"
             "\tshuffled nodes\n"
-        "shuffled_ts (np.ndarray) (noise_repeats, noise_time_steps)\n"
+        "shuffled_ts (np.ndarray) (noise_repeats, noise_bw_it)\n"
             "\tshuffled time series\n"
         #endif
     },
